package ru.dbotthepony.kstarbound.defs.quest

import com.google.common.collect.ImmutableList
import ru.dbotthepony.kommons.io.readCollection
import ru.dbotthepony.kommons.io.writeBinaryString
import ru.dbotthepony.kommons.io.writeCollection
import ru.dbotthepony.kstarbound.io.readInternedString
import ru.dbotthepony.kstarbound.json.builder.JsonFactory
import ru.dbotthepony.kstarbound.network.syncher.legacyCodec
import ru.dbotthepony.kstarbound.network.syncher.nativeCodec
import java.io.DataInputStream
import java.io.DataOutputStream

@JsonFactory
data class QuestArcDescriptor(
	val quests: ImmutableList<QuestDescriptor> = ImmutableList.of(),
	val stagehandUniqueId: String? = null,
) {
	constructor(stream: DataInputStream, isLegacy: Boolean) : this(
		ImmutableList.copyOf(stream.readCollection { QuestDescriptor(this, isLegacy) }),
		if (stream.readBoolean()) stream.readInternedString() else null
	)

	fun write(stream: DataOutputStream, isLegacy: Boolean) {
		stream.writeCollection(quests) { it.write(this, isLegacy) }
		stream.writeBoolean(stagehandUniqueId != null)
		if (stagehandUniqueId != null) stream.writeBinaryString(stagehandUniqueId)
	}

	companion object {
		val CODEC = nativeCodec(::QuestArcDescriptor, QuestArcDescriptor::write)
		val LEGACY_CODEC = legacyCodec(::QuestArcDescriptor, QuestArcDescriptor::write)
	}
}
